@@ -1,5 +1,5 @@
 """
-Copyright (C) 2021-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+Copyright (C) 2021-2025 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
 
 This module provides a tool to download and update clamav databases and database
 patch files (CDIFFs) for the purposes of hosting your own database mirror.
@@ -17,26 +17,47 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from pathlib import Path
 import copy
 import datetime
-from enum import Enum
+import http.client as http_client
 import json
 import logging
 import os
 import platform
-import pkg_resources
 import re
 import subprocess
 import sys
 import time
-from typing import *
 import uuid
-import http.client as http_client
+from enum import Enum
+from pathlib import Path
+from typing import *
 
-from dns import resolver
-import requests
-from requests import status_codes
+try:
+    from importlib.metadata import PackageNotFoundError, version as _get_version
+except ImportError:  # pragma: no cover - backport for older Pythons
+    from importlib_metadata import PackageNotFoundError, version as _get_version
+try:
+    import requests
+except ModuleNotFoundError:
+    class _RequestsMissing:
+        def __getattr__(self, name):
+            raise ModuleNotFoundError(
+                "The 'requests' package is required to perform network operations. "
+                "Install it with 'pip install requests'."
+            )
+    requests = _RequestsMissing()
+try:
+    from dns import resolver
+except ModuleNotFoundError:
+    class _DNSMissing:
+        def __getattr__(self, name):
+            raise ModuleNotFoundError(
+                "The 'dnspython' package is required for DNS lookups. "
+                "Install it with 'pip install dnspython'."
+            )
+    resolver = _DNSMissing()
+from packaging import version
 
 class CvdStatus(Enum):
     NO_UPDATE = 0
@@ -117,7 +138,10 @@ class CVDUpdate:
             db_dir:         path where databases will be downloaded.
             verbose:        Enable DEBUG-level logs and other verbose messages.
         """
-        self.version = pkg_resources.get_distribution('cvdupdate').version
+        try:
+            self.version = _get_version('cvdupdate')
+        except PackageNotFoundError:
+            self.version = "0.0"
         self.verbose = verbose
         self._read_config(
             config,
@@ -130,24 +154,12 @@ class CVDUpdate:
         """
         Initializes the logging parameters.
         """
-        self.logger = logging.getLogger(f"cvdupdate-{self.version}")
-
-        if self.verbose:
-            self.logger.setLevel(logging.DEBUG)
-        else:
-            self.logger.setLevel(logging.INFO)
-
-        formatter = logging.Formatter(
-            fmt="%(asctime)s - %(levelname)s:  %(message)s",
-            datefmt="%Y-%m-%d %I:%M:%S %p",
-        )
-
         today = datetime.datetime.now()
-        self.log_file = self.log_dir / f"{today.strftime('%Y-%m-%d')}.log"
+        log_file = self.log_dir / f"{today:%Y-%m-%d}.log"
 
         if not self.log_dir.exists():
             # Make a new log directory
-            os.makedirs(os.path.split(self.log_file)[0])
+            os.makedirs(log_file.parent)
         else:
             # Log dir already exists, lets check if we need to prune old logs
             logs = self.log_dir.glob('*.log')
@@ -158,11 +170,31 @@ class CVDUpdate:
                     # Log is too old, delete!
                     os.remove(str(log))
 
-        self.filehandler = logging.FileHandler(filename=self.log_file)
-        self.filehandler.setLevel(self.logger.level)
-        self.filehandler.setFormatter(formatter)
+        stderr_level = logging.WARNING
 
-        self.logger.addHandler(self.filehandler)
+        stderr_handler = logging.StreamHandler(sys.stderr)
+        stderr_handler.setLevel(stderr_level)
+
+        class FilterNotStdErr(logging.Filter):
+            def filter(self, record: logging.LogRecord) -> bool:
+                return record.levelno < stderr_level
+
+        stdout_handler = logging.StreamHandler(sys.stdout)
+        stdout_handler.addFilter(FilterNotStdErr())
+
+        logging.basicConfig(
+            level=logging.DEBUG if self.verbose else logging.INFO,
+            format="%(asctime)s - %(levelname)s:  %(message)s",
+            datefmt="%Y-%m-%d %I:%M:%S %p",
+            force=True,  # an import might already have the root logger configured
+            handlers=[
+                stderr_handler,
+                stdout_handler,
+                logging.FileHandler(log_file),
+            ],
+        )
+
+        self.logger = logging.getLogger(f"cvdupdate-{self.version}")
 
         # Also set the log level for urllib3, because it's "DEBUG" by default,
         # and we may not like that.
@@ -309,6 +341,11 @@ class CVDUpdate:
                 try:
                     self.logger.info(f"Deleting: {db}")
                     os.remove(str(cvddb))
+
+                    # If there is a matching .sign digital signature file, remove it too
+                    if os.path.exists(str(cvddb) + ".sign"):
+                        os.remove(str(cvddb) + ".sign")
+
                 except Exception as exc:
                     self.logger.debug(f"Tried to remove {db}")
                     raise exc
@@ -319,8 +356,13 @@ class CVDUpdate:
             try:
                 self.logger.info(f"Deleting CDIFF: {cdiff.name}")
                 os.remove(str(cdiff))
+
+                # If there is a matching .sign digital signature file, remove it too
+                if os.path.exists(str(cdiff) + ".sign"):
+                    os.remove(str(cdiff) + ".sign")
+
             except Exception as exc:
-                self.logger.debug(f"Tried to remove cdiffs.")
+                self.logger.debug(f"Tried to remove CDIFFs.")
 
         # Config cleanup
         for db in dbs:
@@ -361,8 +403,8 @@ class CVDUpdate:
 
         db_paths = self.db_dir.glob('*')
         for db in db_paths:
-            if db.name.endswith('.cdiff'):
-                # Ignore CDIFFs, they'll get printed later.
+            if db.name.endswith('.cdiff') or db.name.endswith('.sign'):
+                # Ignore CDIFFs and sign files, they'll get printed later.
                 continue
 
             if db.name not in dbs:
@@ -548,7 +590,12 @@ class CVDUpdate:
 
         if self.dns_version_tokens == []:
             # Query DNS if we haven't already
-            self._query_dns_txt_entry()
+            for _attempt in range(self.config['max retry']):
+                if self._query_dns_txt_entry():
+                    break
+                # Pause before next attempt.
+                time.sleep(0.1)
+
             if self.dns_version_tokens == []:
                 # Query failed. Bail out.
                 return version
@@ -581,13 +628,14 @@ class CVDUpdate:
         Return 0  if failed.
         '''
         version = 0
-        retry = 0
         url = self.state['dbs'][db]['url']
 
         self.logger.debug(f"Checking {db} version via HTTP download of CVD header.")
 
-        ims = datetime.datetime.utcfromtimestamp(self.state['dbs'][db]['last modified']).strftime('%a, %d %b %Y %H:%M:%S GMT')
+        ims = datetime.datetime.fromtimestamp(self.state['dbs'][db]['last modified'], tz=datetime.timezone.utc).strftime('%a, %d %b %Y %H:%M:%S GMT')
 
+        retry = 0
+        response = None
         while retry < self.config['max retry']:
             response = requests.get(url, headers = {
                 'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
@@ -604,6 +652,9 @@ class CVDUpdate:
                 retry += 1
             else:
                 break
+        if response is None:
+            self.logger.error(f"No response received requesting CVD header from {url}.")
+            return 0
 
         if response.status_code == 200 or response.status_code == 206:
             # Looks like we downloaded something...
@@ -653,9 +704,10 @@ class CVDUpdate:
         Will use If-Modified-Since
         If Not-Modified, it will not replace the current database.
         '''
-        retry = 0
-        ims: str = datetime.datetime.utcfromtimestamp(last_modified).strftime('%a, %d %b %Y %H:%M:%S GMT')
+        ims: str = datetime.datetime.fromtimestamp(last_modified, tz=datetime.timezone.utc).strftime('%a, %d %b %Y %H:%M:%S GMT')
 
+        retry = 0
+        response = None
         while retry < self.config['max retry']:
             response = requests.get(url, headers = {
                 'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
@@ -671,6 +723,9 @@ class CVDUpdate:
                 retry += 1
             else:
                 break
+        if response is None:
+            self.logger.error(f"No response received requesting {url}.")
+            return CvdStatus.ERROR
 
         if response.status_code == 200:
             # Looks like we downloaded something...
@@ -702,6 +757,15 @@ class CVDUpdate:
             # Not modified since IMS. We have the latest version.
             version = self.state['dbs'][db]['local version']
             self.logger.info(f"{db} not-modified since: {ims} (local version {version})")
+
+            # Check for .cvd.sign.
+            # it won't download if we already have it.
+            self._download_sign_file_for(
+                db,
+                url,
+                last_modified=0,
+                version=version)
+
             return CvdStatus.NO_UPDATE
 
         elif response.status_code == 429:
@@ -726,6 +790,195 @@ class CVDUpdate:
             self.logger.error(f"Failed to download {db} from {url}")
             return CvdStatus.ERROR
 
+        # Now try downloading the corresponding .cvd.sign.
+        # It's okay if it doesn't exist
+        self._download_sign_file_for(
+            db,
+            url,
+            last_modified=0,
+            version=version)
+
+        return CvdStatus.UPDATED
+
+    def _download_cdiff(self, db: str, file: str, db_url: str, last_modified: int, desired_version: int, available_version: int) -> CvdStatus:
+        '''
+        Download a CDIFF file given a file name and version.
+        The file name should be in the format of "daily-12345.cdiff"
+        '''
+
+        self.logger.debug(f"Checking for {file}")
+
+        # now remove the old file name from the db_url and add the new sign file name
+        base_url = db_url.rsplit('/', 1)[0]
+        url = f"{base_url}/{file}"
+
+        retry = 0
+        response = None
+        while retry < self.config['max retry']:
+            response = requests.get(url, headers = {
+                'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
+            })
+
+            if ((response.status_code == 200 or response.status_code == 206) and
+                ('content-length' in response.headers) and
+                (int(response.headers['content-length']) > len(response.content))):
+                self.logger.warning(f"Response was truncated somehow...")
+                self.logger.warning(f"   Expected {response.headers['content-length']}")
+                self.logger.warning(f"   Received {response.content}, let's retry.")
+                retry += 1
+            else:
+                break
+        if response is None:
+            self.logger.error(f"No response received requesting {url}.")
+            return CvdStatus.ERROR
+
+        if response.status_code == 200:
+            # Looks like we downloaded something...
+            if (('content-length' in response.headers) and int(response.headers['content-length']) > len(response.content)):
+                self.logger.error(f"Failed to download CDIFF.")
+                return CvdStatus.ERROR
+
+            # Download Success
+            self.logger.info(f"Downloaded {file}")
+            try:
+                with (self.db_dir / f"{file}").open('wb') as new_db:
+                    new_db.write(response.content)
+            except Exception as exc:
+                self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
+                self.logger.error(f"Failed to save {file} to {self.db_dir}.")
+
+            # Update config with CDIFF, for posterity
+            self.state['dbs'][db]['CDIFFs'].append(file)
+
+            # Prune old CDIFFs if needed
+            if len(self.state['dbs'][db]['CDIFFs']) > self.config['# cdiffs to keep']:
+                try:
+                    os.remove(self.db_dir / self.state['dbs'][db]['CDIFFs'][0])
+                except Exception as exc:
+                    self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
+                    self.logger.debug(f"Tried to prune old cdiffs, but they weren't found, maybe someone else removed them already.")
+
+                self.state['dbs'][db]['CDIFFs'] = self.state['dbs'][db]['CDIFFs'][1:]
+
+        elif response.status_code == 429:
+            # Rejected because downloading the same file too frequently.
+            self.logger.warning(f"Failed to download {file}")
+            self.logger.warning(f"Download request rejected because we've downloaded the same file too frequently.")
+
+            try_again_seconds = 60 * 60 * 12 # 12 hours
+            if 'Retry-After' in response.headers.keys():
+                try_again_seconds = int(response.headers['Retry-After'])
+
+            self.state['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
+
+            try_again_string = str(datetime.timedelta(seconds=try_again_seconds))
+            self.logger.warning(f"We won't try {db} again for {try_again_string} hours.")
+
+            # Sure only a CDIFF failed, but if we want any chance of trying the CDIFF again
+            # in the future, let's bail out now and retry the CVD + CDIFFs after the cooldown.
+            return CvdStatus.ERROR
+
+        else:
+            # HTTP Get failed.
+            self.logger.info(f"No CDIFF found for {db} version # {desired_version}")
+
+            if desired_version < available_version:
+                desired_version = available_version - 1
+                self.logger.info(f"Will just skip to the last CDIFF instead.")
+            else:
+                self.logger.info(f"Giving up on CDIFFs for {db}")
+                return CvdStatus.NO_UPDATE
+
+        return CvdStatus.UPDATED
+
+
+    def _download_sign_file_for(self, file: str, file_url: str, last_modified: int, version=0) -> CvdStatus:
+        '''
+        Download signature file given a file name.
+        If version > 0, will ensure sign file includes version in the filename, like this:
+        - file-version.ext.sign
+        '''
+        ims: str = datetime.datetime.fromtimestamp(last_modified, tz=datetime.timezone.utc).strftime('%a, %d %b %Y %H:%M:%S GMT')
+
+        sign_file = file + ".sign"
+        if version > 0 and str(version) not in file:
+            # the sign file name should include the version in this format: "file-version.ext.sign"
+            # reconstruct.
+            name_parts = file.rsplit('.', 1)
+            if len(name_parts) == 1:
+                self.logger.error(f"Invalid file name. Lacks extension: {file}")
+                return CvdStatus.ERROR
+
+            file_name = name_parts[0]
+            ext = name_parts[-1]
+            sign_file = f"{file_name}-{version}.{ext}.sign"
+
+        # check if we already have it.
+        if (self.db_dir / sign_file).exists():
+            self.logger.debug(f"We already have {sign_file}. Skipping...")
+            return CvdStatus.NO_UPDATE
+
+        # now remove the old file name from the file_url and add the new sign file name
+        base_url = file_url.rsplit('/', 1)[0]
+        url = f"{base_url}/{sign_file}"
+
+        retry = 0
+        response = None
+        while retry < self.config['max retry']:
+            response = requests.get(url, headers = {
+                'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
+                'If-Modified-Since': ims,
+            })
+
+            if ((response.status_code == 200 or response.status_code == 206) and
+                ('content-length' in response.headers) and
+                (int(response.headers['content-length']) > len(response.content))):
+                self.logger.warning(f"Response was truncated somehow...")
+                self.logger.warning(f"   Expected {response.headers['content-length']}")
+                self.logger.warning(f"   Received {response.content}, let's retry.")
+                retry += 1
+            else:
+                break
+        if response is None:
+            self.logger.error(f"No response received requesting {url}.")
+            return CvdStatus.ERROR
+
+        if response.status_code == 200:
+            # Looks like we downloaded something...
+            if (('content-length' in response.headers) and int(response.headers['content-length']) > len(response.content)):
+                self.logger.error(f"Failed to download {sign_file}")
+                return CvdStatus.ERROR
+
+            # Download Success
+            if version > 0:
+                self.logger.info(f"Downloaded {sign_file}. Version: {version}")
+            else:
+                self.logger.info(f"Downloaded {sign_file}")
+
+            try:
+                with (self.db_dir / sign_file).open('wb') as new_db:
+                    new_db.write(response.content)
+
+            except Exception as exc:
+                self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
+                self.logger.error(f"Failed to save {sign_file} to {self.db_dir}")
+                return CvdStatus.ERROR
+
+        elif response.status_code == 304:
+            # Not modified since IMS. We have the latest version.
+            self.logger.info(f"{sign_file} not-modified since: {ims} (local version {version})")
+            return CvdStatus.NO_UPDATE
+
+        elif response.status_code == 429:
+            # Rejected because downloading the same file too frequently.
+            self.logger.warning(f"Failed to download {sign_file} from {url} with 429 response.")
+            return CvdStatus.ERROR
+
+        else:
+            # HTTP Get failed.
+            self.logger.info(f"Request failed for {url}. Probably no external digital signature provided for {file}")
+            return CvdStatus.ERROR
+
         return CvdStatus.UPDATED
 
     def _download_cvd(self, db: str, available_version: int) -> CvdStatus:
@@ -740,6 +993,17 @@ class CVDUpdate:
         if local_version >= available_version:
             # Oh! We're already up to date, don't worry about it.
             self.logger.info(f"{db} is up-to-date. Version: {local_version}")
+
+            db_url = self.state['dbs'][db]['url']
+
+            # Check for the .cvd.sign file, just in case we don't have that yet.
+            # It won't download if we already have it.
+            self._download_sign_file_for(
+                db,
+                db_url,
+                last_modified=0,
+                version=available_version)
+
             return CvdStatus.NO_UPDATE
 
         elif local_version == 0:
@@ -749,96 +1013,41 @@ class CVDUpdate:
         # First try to get CDIFFs
         self.logger.debug(f"Downloading CDIFFs first...")
         while desired_version <= available_version:
-            # Attempt to download each CDIFF betwen our local version and the available version
+            # Attempt to download each CDIFF between our local version and the available version.
             # The url for CVDs should be https://database.clamav.net/<db>
             # Eg:
             #   https://database.clamav.net/daily.cvd
-            # For the daily cdiffs, we would want:
+            # For the daily CDIFFs, we would want:
             #   https://database.clamav.net/daily-<version>.cdiff
-            retry = 0
-            cdiff_filename = f"{db[:-len('.cvd')]}-{desired_version}.cdiff"
+            cdiff_file = f"{db[:-len('.cvd')]}-{desired_version}.cdiff"
 
-            original_url = self.state['dbs'][db]['url']
-            url = f"{original_url[:-len(db)]}{cdiff_filename}"
-
-            if (self.db_dir / cdiff_filename).exists():
-                self.logger.debug(f"We already have {cdiff_filename}. Skipping...")
+            if (self.db_dir / cdiff_file).exists():
+                self.logger.debug(f"We already have {cdiff_file}. Skipping...")
                 desired_version += 1
                 continue
 
-            self.logger.debug(f"Checking for {cdiff_filename}")
+            db_url = self.state['dbs'][db]['url']
 
-            while retry < self.config['max retry']:
-                response = requests.get(url, headers = {
-                    'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
-                })
+            # Download the .cdiff
+            result = self._download_cdiff(
+                db,
+                cdiff_file,
+                db_url,
+                last_modified=0,
+                desired_version=desired_version,
+                available_version=available_version)
 
-                if ((response.status_code == 200 or response.status_code == 206) and
-                    ('content-length' in response.headers) and
-                    (int(response.headers['content-length']) > len(response.content))):
-                    self.logger.warning(f"Response was truncated somehow...")
-                    self.logger.warning(f"   Expected {response.headers['content-length']}")
-                    self.logger.warning(f"   Received {response.content}, let's retry.")
-                    retry += 1
-                else:
-                    break
+            if result != CvdStatus.UPDATED:
+                self.logger.error(f"Failed to download {cdiff_file}.")
+                break
 
-            if response.status_code == 200:
-                # Looks like we downloaded something...
-                if (('content-length' in response.headers) and int(response.headers['content-length']) > len(response.content)):
-                    self.logger.error(f"Failed to download {db} header to check the version #.")
-                    return CvdStatus.ERROR
-
-                # Download Success
-                self.logger.info(f"Downloaded {cdiff_filename}")
-                try:
-                    with (self.db_dir / f"{cdiff_filename}").open('wb') as new_db:
-                        new_db.write(response.content)
-                except Exception as exc:
-                    self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
-                    self.logger.error(f"Failed to save {cdiff_filename} to {self.db_dir}.")
-
-                # Update config with CDIFF, for posterity
-                self.state['dbs'][db]['CDIFFs'].append(cdiff_filename)
-
-                # Prune old CDIFFs if needed
-                if len(self.state['dbs'][db]['CDIFFs']) > self.config['# cdiffs to keep']:
-                    try:
-                        os.remove(self.db_dir / self.state['dbs'][db]['CDIFFs'][0])
-                    except Exception as exc:
-                        self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
-                        self.logger.debug(f"Tried to prune old cdiffs, but they weren't found, maybe someone else removed them already.")
-
-                    self.state['dbs'][db]['CDIFFs'] = self.state['dbs'][db]['CDIFFs'][1:]
-
-            elif response.status_code == 429:
-                # Rejected because downloading the same file too frequently.
-                self.logger.warning(f"Failed to download {cdiff_filename}")
-                self.logger.warning(f"Download request rejected because we've downloaded the same file too frequently.")
-
-                try_again_seconds = 60 * 60 * 12 # 12 hours
-                if 'Retry-After' in response.headers.keys():
-                    try_again_seconds = int(response.headers['Retry-After'])
-
-                self.state['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
-
-                try_again_string = str(datetime.timedelta(seconds=try_again_seconds))
-                self.logger.warning(f"We won't try {db} again for {try_again_string} hours.")
-
-                # Sure only a CDIFF failed, but if we want any chance of trying the CDIFF again
-                # in the future, let's bail out now and retry the CVD + CDIFFs after the cooldown.
-                return CvdStatus.ERROR
-
-            else:
-                # HTTP Get failed.
-                self.logger.info(f"No CDIFF found for {db} version # {desired_version}")
-
-                if desired_version < available_version:
-                    desired_version = available_version - 1
-                    self.logger.info(f"Will just skip to the last CDIFF instead.")
-                else:
-                    self.logger.info(f"Giving up on CDIFFs for {db}")
-                    break
+            # Now try downloading the corresponding .cdiff.sign.
+            # It's okay if it doesn't exist.
+            self._download_sign_file_for(
+                cdiff_file,
+                db_url,
+                last_modified=0,
+                version=desired_version)
 
             desired_version += 1
 
@@ -892,33 +1101,37 @@ class CVDUpdate:
 
     def pypi_update_check(self):
         def check(name):
-            '''
-            Check if there's a newer version of the cvdupdate package.
-            From https://stackoverflow.com/questions/58648739/how-to-check-if-python-package-is-latest-version-programmatically
-            '''
-            self.logger.debug(f'Checking for a newer version of cvdupdate.')
+            """Checks if a newer version of the specified module is available on PyPI."""
+            self.logger.debug(f'Checking for a newer version of {name}.')
+            try:
+                current_version_str = _get_version(name)
+                current_version = version.parse(current_version_str)
 
-            result = subprocess.run([sys.executable, '-m', 'pip', 'install', '{}==random'.format('cvdupdate')], stdout=subprocess.DEVNULL, stderr=subprocess.PIPE)
-            latest_version = result.stderr.decode("utf-8")
-            latest_version = latest_version[latest_version.find('(from versions:')+15:]
-            latest_version = latest_version[:latest_version.find(')')]
-            latest_version = latest_version.replace(' ','').split(',')[-1].strip()
+                response = requests.get(f"https://pypi.org/pypi/{name}/json")  # Get package info
+                response.raise_for_status() # Raise an exception for bad status codes (4xx or 5xx)
+                latest_version_str = response.json()["info"]["version"]
+                latest_version = version.parse(latest_version_str)
 
-            result = subprocess.run([sys.executable, '-m', 'pip', 'show', '{}'.format('cvdupdate')], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
-            current_version = result.stdout.decode("utf-8")
-            current_version = current_version[current_version.find('Version:')+8:]
-            current_version = current_version[:current_version.find('\n')].replace(' ','').strip()
+                if latest_version > current_version:
+                    self.logger.warning(f'You are running {name} version: {current_version}.')
+                    self.logger.warning(f'There is a newer version on PyPI: {latest_version}. Please update!')
+                    return False  # Newer version available
 
-            if 'ERROR' in latest_version:
-                self.logger.debug(f"Version check didn't work, didn't get back a list of package versions.")
-                return True
-            elif latest_version == current_version:
-                self.logger.debug(f'cvdupdate is up-to-date: {current_version}.')
-                return True
-            else:
-                self.logger.warning(f'You are running cvdupdate version: {current_version}.')
-                self.logger.warning(f'There is a newer version on PyPI: {latest_version}. Please update!')
-                return False
+                else:
+                    self.logger.debug(f'{name} is up-to-date: {current_version}.')
+                    return True  # No newer version
+
+            except requests.exceptions.RequestException as e:
+                self.logger.debug(f"Version check failed: {e}")  # Log the error
+                return True  # Assume up-to-date on error
+
+            except PackageNotFoundError:
+                self.logger.error(f"Package {name} not found locally.")
+                return True  # Assuming not found means no update possible
+
+            except Exception as e:  # Catch other potential errors (json parsing etc.)
+                self.logger.debug(f"Version check failed: {e}")
+                return True  # Assume up-to-date on error
 
         return check('cvdupdate')
 
